=begin
dbStair Builder provides the raw geometry for a 'residential' staircase complying with UK Building Regulations. The script will calculate and draw the correct number and size of steps / risers for the specified total height (floor to floor). User can specify upto 3 flights seperated by either landings or winders. The 3 kite winder constructed in this script is optimised for a stair width of 900mm, tread 'going' of approx 220mm and riser of approx 200mm. UK Building Regulations require that the 'going' for all treads in a staircase (including winders) are the same. Whilst this script will construct winder staircases with varying widths and tread dimensions, they may not be compliant. The staircase can be constructed with stringers but these may need tweaking to 'look nice'! Handrails, balustrades etc are left to the user to complete. 

Instructions:-
1. To locate the staircase, draw ANY non-equal sided right-angled triangle (e.g a 3,4,5 sided triangle), with the right angle corner on the left side at the start of the staircase (outside of staircase stringer, in line with front of first step nosing). The triangle should be orientated so that the short right-angled side, lines up with the first riser, and the longer right-angle side on the left of the staircase, in-line with the direction of the first flight.
2.	Select the triangle face only, and click on dbStaircase Builder. If no triangle face is selected, dbStaircase Builder will
present a warning box saying the staircase will be constructed at the origin [0,0,0].
3.	Adjust the parameters in the dialogue box.
	* Total staircase height - Floor to floor height (if zero is input, the script will not restrict number of steps, and will present another dialogue box for user to input tread and riser dimensions).
	* Staircase width - (including stringers).
	* Number of steps in each flight.
	* Turn type - 'None' will build one straight flight.
				  'Landings' will build flat platform between flights
				  'Winders' will build a 3 kite winder (most space efficient winder box) between flights.
	* Stair Hand - Left or Right.
	* Stringers - "On" or "Off" whether stringers will be constructed.
	* Restrict steps to total height - 'Yes' will prompt to adjust number of steps in staircase to fit floor to floor height.
	                                   'No' will build whatever steps are specified without regard to floor to floor height.

Notes:-
* Treads, risers (apart from first riser) and stringers are components for speed of editing.
* Final trimming of stringers, filleting treads etc are left to the user).

=end  
module DeanBy7
	module Stair_Builder
		def self.main
		    @tht = 2600.mm if(!@tht)
			@wid = 900.mm if(!@wid)
			@trd = 32.mm if(!@trd)
			@ris = 12.mm if(!@ris)
			@nos = 20.mm if(!@nos)
			@stg = 32.mm if(!@stg)
			@stgd = 240.mm if(!@stgd)
			@flt1 = 13 if(!@flt1)
			@flt2 = 0 if(!@flt2)
			@flt3 = 0 if(!@flt3)
			@turns = "None" if(!@turns)
			@hand = "Left" if(!@hand)
            @restrict = "Yes" if(!@restrict)
			@strgon = "On" if(!@strgon)
			org = ORIGIN
			@mod = Sketchup.active_model
            sel = @mod.selection
			if (sel[0].is_a? (Sketchup::Face))
			    verts = sel[0].vertices
				v1 = verts[0].position
				v2 = verts[1].position
				v3 = verts[2].position
				l12 = v1.distance v2
				l23 = v2.distance v3
				l13 = v1.distance v3
				if (l12 > l23)
				    if (l12 > l13)
					    hyp = l12
					end
				else
				    if (l23 > l13)
					    hyp = l23
					else
					    hyp = l13
					end
				end
				
				if (hyp == l12)
				    sto = v3
					if ((v3.distance v2) < (v3.distance v1))
					    stf = v2
					else
					    stf = v1
					end
 
				elsif (hyp == l13)
				    sto = v2
					if ((v2.distance v1) < (v2.distance v3))
					    stf = v1
					else
					    stf = v3
					end

				else
				    sto = v1
					if ((v1.distance v2) < (v1.distance v3))
					    stf = v2
					else
					    stf = v3
					end
				end			

			else
			    UI.messagebox "Staircase will be located at 0,0,0"
			end


                  				
			xa = X_AXIS; ya = Y_AXIS; za = Z_AXIS
			ents = @mod.entities
			str_grp = ents.add_group()
			str_grp.name = "dbStairs"
			@sents = str_grp.entities
			go = true			
			while go
			    prompts = ["Stair Height Total", "Stair Width (inc Stringers)","First Flight Steps","Second Flight Steps","Third Flight Steps","Turn Type","Stair Hand","Stringers","Restrict Steps to total height?"]
			    defaults = [@tht,@wid,@flt1,@flt2,@flt3,@turns,@hand,@strgon,@restrict]

			    list = ["", "", "", "", "", "None|Landings|Winders","Left|Right","On|Off","Yes|No"]
			    results = UI.inputbox prompts, [@tht,@wid,@flt1,@flt2,@flt3,@turns,@hand,@strgon,@restrict], list, "db Stair Builder"
			    @tht,@wid,@flt1,@flt2,@flt3,@turns,@hand,@strgon,@restrict = results
		        @turn1 = @turns
				@turn2 = @turns
				if (@flt3 == 0)
				    @turn2 = "None"
				end

			    @mod.start_operation "db Window Opening"
				
				if (@tht > 0)	
					testrisers = @tht / 200.mm
					@risers = testrisers.ceil
					
					@riser = (@tht / @risers)
					@tread = (@riser / Math.tan(42.degrees))

				else
					@restrict = "No"
					trgo = true
					while trgo
						@tread = 223.mm
						@riser = 200.mm
						prompts = ["Tread Going","Tread Riser"]
						defaults = [@tread,@riser]
						results = UI.inputbox prompts, [@tread,@riser], "Manual tread input"
						@tread,@riser = results
						@risers = @flt1
						if (@tread > 0) && (@riser > 0)
							trgo = false
						end
					end
				end
				
			    if (@turn1 == "None") ;@turn1v = 0 ;end
			    if (@turn1 == "Landings") ;@turn1v = 1 ;end
			    if (@turn1 == "Winders") ;@turn1v = 3 ;end
			    if (@turn2 == "None") ;@turn2v = 0 ;end
			    if (@turn2 == "Landings") ;@turn2v = 1 ;end
			    if (@turn2 == "Winders") ;@turn2v = 3 ;end
			    if (@turns == "None")
			        @flt2 = 0
				    @flt3 = 0
		        end
				tusteps = @flt1 + @turn1v + @flt2 + @turn2v + @flt3
				#half landing adjustment of test
				if (@turn1 == "Landings") && (@turn2 == "Landings") && (@flt2 == 0)
				    tusteps = tusteps - 1
				end
				if (@restrict == "Yes")
			        if (tusteps != @risers)
					    if (tusteps > @risers)						    
				            over = tusteps - @risers
                            UI.messagebox "Total steps needed " + @risers.to_s + "\n\n" + over.to_s + " too many steps specified"
					    elsif (tusteps < @risers)
						    under = @risers - tusteps
					        UI.messagebox "Total steps needed " + @risers.to_s + "\n\n" + under.to_s + " too few steps specified"
						end
					else
					    go = false
					end
                else
			        go = false 				
				end
               				
			end
			
			
			@width = @wid - (@stg * 2)
			@scf = 1			
			
			#scale factor for non 900mm wide staircase with winders.
			if (@turns == "Winders") && (@wid !=900.mm)
			    @scf = @wid / 900.mm
            end                				
			
			
			self.define_step()
			self.define_rise()
			
            if (@flt1 > 0)
			
			    @ft1_grp = @sents.add_group()
			    @ft1ents = @ft1_grp.entities
		        @ft1_grp.name = "flight1"
                #first riser
		        p0 = ORIGIN
		        r0 = p0.offset(xa, @stg)
				r0.offset!(ya, @nos)
		        r1 = r0.offset(xa, @width)
		        r2 = r1.offset(za, @riser-@trd)
		        r3 = r2.offset(xa, -@width)
		        rise1 = @ft1ents.add_face(r0,r1,r2,r3)
		        rise1.pushpull -@ris
				
                #place steps			
			    @pt = ORIGIN + [0,0,@riser]
				@ptt1 = @pt.offset(za, 2000.mm)
				@ptt3 = @ptt1.offset(xa, @wid)

			    @pt.offset!(xa, @stg)
                for i in (1..@flt1)
                    t = Geom::Transformation.new( @pt )
                    @ft1ents.add_instance( @step_def, t )
				    @pt += [0, @tread, @riser]
                end
				#hidden lines for headroom
				@ptt2 = @pt.offset(za, 2000.mm)
				@ptt4 = @ptt2.offset(xa, @wid)
				flt1hdr1 = @ft1ents.add_line(@ptt1,@ptt2)
				flt1hdr2 = @ft1ents.add_line(@ptt3,@ptt4)
				flt1hdr1.hidden = true
				flt1hdr2.hidden = true
				
			    #place risers
			    @pt = ORIGIN + [0,@tread,@riser -@trd]
			    @pt.offset!(ya, @nos)
                @pt.offset!(xa, @stg)					
			    for i in (1...@flt1)
                    t = Geom::Transformation.new( @pt )
                    @ft1ents.add_instance( @rise_def, t )
				    @pt += [0, @tread, @riser]
                end
				#New strings
				if (@strgon == "On")
					s0 = ORIGIN
					s1 = s0.offset(za, -215.mm)
					s1.offset!(ya, -85.mm)
					s2 = s0.offset(za, -144.3.mm)
					s2.offset!(ya, -163.6.mm)
					s3 = s0.offset(za, 25.mm)
					s3.offset!(ya, -11.1.mm)
					pilen = (@tread**2 + @riser**2) ** 0.5
					s4 = s0.offset(za, 25.mm)
					s4.offset!(ya, (@flt1 * pilen) + 65.2.mm)
					s5 = s0.offset(za, -215.mm)
					s5.offset!(ya, (@flt1 * pilen) - 150.9.mm)
					def_list = @mod.definitions
					@strg_def = @def_list.add "Stringers1"
					cdents = @strg_def.entities
					strg = cdents.add_face(s1,s2,s3,s4,s5)
					strg.pushpull -@stg
					@angle = Math.atan(@riser / @tread)
					tl1 = Geom::Transformation.translation([0,0,@riser])
					tl2 = Geom::Transformation.translation([@width + @stg,0,@riser])
					tr = Geom::Transformation.rotation([0,0,@riser],[1,0,0],@angle)
					t1 = tr * tl1
					t2 = tr * tl2
					inst1 = @ft1ents.add_instance @strg_def, t1					
					inst2 = @ft1ents.add_instance @strg_def, t2
				end
			else
			    @pt = ORIGIN				
				p0 = ORIGIN
				pw1 = @pt
				@pt2 = pw1
			end
			
		    if (@turn1=="Winders")
                self.winders()
			end
			
			if (@turn1=="Landings")
                @land1_grp = @sents.add_group()
			    @ld1ents = @land1_grp.entities
			    @land1_grp.name = "Landing1"
				
                @lstep_grp = @ld1ents.add_group()
			    @lsents = @lstep_grp.entities
			    @lstep_grp.name = "L1step"				

			    p1 = p0.offset(xa, @width)
				q1 = p1.offset(za, @stg + 62.2.mm)
				q1.offset!(ya, 32.mm)
				q6 = q1.offset(za, -323.mm)
			    p2 = p1.offset(ya, @wid + @nos + 1.5.mm)
				q2 = p2.offset(za, @stg + 62.2.mm)
				q5 = q2.offset(za, -323.mm)				
				q2a = q2.offset(ya, @stg)
				q5a = q2a.offset(za, -323.mm)
				p3 = p2.offset(xa, -@width - 53.5.mm)
				q23 = q2.offset(xa, -662.mm)
				q23u = q23.offset(za, -323.mm)
				q3 = p3.offset(za, @stg + 268.3.mm) # 62.2
				q4 = q3.offset(za, -323.mm)
                p4 = p0.offset(ya, 107.mm)
                p5 = p4.offset(xa, -53.5.mm)
                if (@flt2 == 0) && (@turn2 == "Landings")
				    #half landing
					p3h = p2.offset(xa, -@width - 53.5.mm - @wid)
					q3h = p3h.offset(za, @stg + 62.2.mm)
					q3ha = q3h.offset(ya, @stg)
					q3hu = q3h.offset(za, -323.mm)
					q3hua = q3ha.offset(za, -323.mm)
					p5h = p3h.offset(ya, -@wid - @nos + @stg)
					q5h = p5h.offset(za, @stg + 62.2.mm)
					q5hu = q5h.offset(za, -323.mm)
					p4h = p0.offset(ya, 32.mm)
					lstep = @lsents.add_face(p0,p1,p2,p3h,p5h,p4h)
			        lstep.pushpull -@trd
					if (@strgon == "On")
						@lstg1_grp = @ld1ents.add_group()
						@ls1ents = @lstg1_grp.entities
						@lstg1_grp.name = "L1string1"
						lstg1 = @ls1ents.add_face(q1,q2a,q5a,q6)
						lstg1.pushpull -@stg
					
						@lstg2_grp = @ld1ents.add_group()
						@ls2ents = @lstg2_grp.entities
						@lstg2_grp.name = "L1string2"					
						lstg2 = @ls2ents.add_face(q2,q3h,q3hu,q5)
						lstg2.pushpull -@stg

						@lstg3_grp = @ld1ents.add_group()
						@ls3ents = @lstg3_grp.entities
						@lstg3_grp.name = "L1string1"

						lstg3 = @ls3ents.add_face(q3ha,q5h,q5hu,q3hua)
						lstg3.pushpull -@stg
					end
				else
			        lstep = @lsents.add_face(p0,p1,p2,p3,p5,p4)
			        lstep.pushpull -@trd
					if (@strgon == "On")
						@lstg1_grp = @ld1ents.add_group()
						@ls1ents = @lstg1_grp.entities
						@lstg1_grp.name = "L1string1"
						lstg1 = @ls1ents.add_face(q1,q2a,q5a,q6)
						lstg1.pushpull -@stg

						@lstg2_grp = @ld1ents.add_group()
						@ls2ents = @lstg2_grp.entities
						@lstg2_grp.name = "L1string2"					
						lstg2 = @ls2ents.add_face(q2,q23,q3,q4,q23u,q5)
						lstg2.pushpull -@stg
					end
				end
			    pw1 = @pt.offset(ya, - @nos)
			    pw1.offset!(za, @riser)
				@lstep_grp.move! pw1
				if (@strgon == "On")
					@lstg1_grp.move! pw1
					@lstg2_grp.move! pw1
					if (@flt2 == 0) && (@turn2 == "Landings")
						@lstg3_grp.move! pw1
					end
				end
			    pr1 = pw1.offset(ya, @nos)
			    pr1.offset!(za, -@riser)
			    inst = @ld1ents.add_instance @rise_def, pr1
				
			    p1 = p0.offset(xa, (75.mm - @stg) / 2)
			    p1.offset!(ya, 32.mm)
			    p2 = p1.offset(ya, 75.mm)
			    p3 = p2.offset(xa, -75.mm)
			    p4 = p3.offset(ya, -75.mm)
			    post_grp = @ld1ents.add_group()
			    post_grp.name = "Post"
			    pents = post_grp.entities
			    post = pents.add_face(p1,p2,p3,p4)
			    post.pushpull @riser * 3
			    pw1.offset!(za, @riser)
			    post_grp.move! pw1
				pp1 = pw1.offset(za, 100.mm)
				post_grp.move! pp1
			    @pt2 = pw1
				pw2 = pw1.offset(xa, -87.mm)
                if (@flt2 == 0) && (@turn2 == "Landings")
                    post2_grp = post_grp.copy
                    post2_grp.name = "post2"
                    post2_grp.move! (pw2)					
				end				
			end    
			
            if (@flt2 > 0)

			    @ft2_grp = @sents.add_group()
			    @ft2ents = @ft2_grp.entities
		        @ft2_grp.name = "flight2"
				
                #place steps			
			    @pt = ORIGIN + [0,0,@riser]
				
			    @pt.offset!(xa, @stg)
                for i in (1..@flt2)
                    t = Geom::Transformation.new( @pt )
                    @ft2ents.add_instance( @step_def, t )
				    @pt += [0, @tread, @riser]
                end
				
			    #place risers
			    @pt = ORIGIN + [0,0, -@trd]
			    @pt.offset!(ya, @nos)
                @pt.offset!(xa, @stg)					
			    for i in (1..@flt2)
                    t = Geom::Transformation.new( @pt )
                    @ft2ents.add_instance( @rise_def, t )
				    @pt += [0, @tread, @riser]
                end
                #new stringers
				if (@strgon == "On")
					s0 = ORIGIN
					s1 = s0.offset(za, 25.mm)
					s1.offset!(ya, 49.4.mm)
					s2 = s0.offset(za, -215.mm)
					s2.offset!(ya, -166.7.mm)
					s3 = s0.offset(za, 25.mm)
					pilen = (@tread**2 + @riser**2) ** 0.5
					s3 = s0.offset(za, 25.mm)
					s3.offset!(ya, (@flt2 * pilen) + 65.2.mm)
					s4 = s0.offset(za, -215.mm)
					s4.offset!(ya, (@flt2 * pilen) - 150.9.mm)
					def_list = @mod.definitions
					@strg_def = @def_list.add "Stringers2"
					cdents = @strg_def.entities
					strg = cdents.add_face(s2,s1,s3,s4)
					strg.pushpull -@stg
					@angle = Math.atan(@riser / @tread)
					tl1 = Geom::Transformation.translation([0,0,@riser])
					tl2 = Geom::Transformation.translation([@width + @stg,0,@riser])
					tr = Geom::Transformation.rotation([0,0,@riser],[1,0,0],@angle)
					t1 = tr * tl1
					t2 = tr * tl2
					inst1 = @ft2ents.add_instance @strg_def, t1					
					inst2 = @ft2ents.add_instance @strg_def, t2		    
				end				
				#move flight 2
				@pt2.offset!(za,(-@riser + @trd))
				@pt2.offset!(ya, 53.5.mm)
				@pt2.offset!(xa, -33.5.mm)
			    tr = Geom::Transformation.rotation([0,0,0],[0,0,1],90.degrees)
			    tl = Geom::Transformation.new( @pt2 )
			    tt = tl * tr
			    @ft2_grp.transform! tt
				
			end


			if (@turn2 == "Winders")
			    @wbox2_grp = @wbox_grp.copy
			    @wbox2_grp.name = "winderbox2"
			    tr = Geom::Transformation.rotation([0,(@flt1*@tread),0],[0,0,1],90.degrees)
				tl = Geom::Transformation.translation([-(@flt2*@tread)-1.5.mm,53.5.mm,(@riser*3)+(@flt2*@riser)] )
			    tt = tl * tr
			 	@wbox2_grp.transform! tt
				if (@flt1 == 0)
				    tl = Geom::Transformation.translation([-@stg - @nos,@ris,0])
                    @wbox2_grp.transform! tl					
				end
			end

			if (@turn2 == "Landings")
			    if (@flt2 != 0)
			        @land2_grp = @land1_grp.copy
			        @land2_grp.name = "landing2"
			        tr = Geom::Transformation.rotation([0,(@flt1*@tread),0],[0,0,1],90.degrees)
				    tl = Geom::Transformation.translation([-(@flt2*@tread)-1.5.mm,53.5.mm,@riser+(@flt2*@riser)] )
			        tt = tl * tr
			 	    @land2_grp.transform! tt
				    if (@flt1 == 0)
				        tl = Geom::Transformation.translation([-@stg - @nos,@ris,0])
                        @land2_grp.transform! tl					
				    end
				end
			end
			
			
            if (@flt3 > 0)
			    @ft3_grp = @sents.add_group()
			    @ft3ents = @ft3_grp.entities
		        @ft3_grp.name = "flight3"
                #place steps			
			    @pt = ORIGIN + [0,0,@riser]
			    @pt.offset!(xa, @stg)
                for i in (1..@flt3)
                    t = Geom::Transformation.new( @pt )
                    @ft3ents.add_instance( @step_def, t )
				    @pt += [0, @tread, @riser]
                end
				
			    #place risers
			    @pt = ORIGIN + [0,0, -@trd]
			    @pt.offset!(ya, @nos)
                @pt.offset!(xa, @stg)					
			    for i in (1..@flt3)
                    t = Geom::Transformation.new( @pt )
                    @ft3ents.add_instance( @rise_def, t )
				    @pt += [0, @tread, @riser]
                end

                #new stringers
				if (@strgon == "On")
					s0 = ORIGIN
					s1 = s0.offset(za, 25.mm)
					s1.offset!(ya, 49.4.mm)
					s2 = s0.offset(za, -215.mm)
					s2.offset!(ya, -166.7.mm)
					s3 = s0.offset(za, 25.mm)
					pilen = (@tread**2 + @riser**2) ** 0.5
					s3 = s0.offset(za, 25.mm)
					s3.offset!(ya, (@flt3 * pilen) + 65.2.mm)
					s4 = s0.offset(za, -215.mm)
					s4.offset!(ya, (@flt3 * pilen) - 150.9.mm)
					def_list = @mod.definitions
					@strg_def = @def_list.add "Stringers3"
					cdents = @strg_def.entities
					strg = cdents.add_face(s2,s1,s3,s4)
					strg.pushpull -@stg
					@angle = Math.atan(@riser / @tread)
					tl1 = Geom::Transformation.translation([0,0,@riser])
					tl2 = Geom::Transformation.translation([@width + @stg,0,@riser])
					tr = Geom::Transformation.rotation([0,0,@riser],[1,0,0],@angle)
					t1 = tr * tl1
					t2 = tr * tl2
					inst1 = @ft3ents.add_instance @strg_def, t1					
					inst2 = @ft3ents.add_instance @strg_def, t2		    
				end
				if (@turn1 == "Landings") && (@flt2 == 0) && (@turn2 == "Landings")
				    topz = @flt1 + 1
				else
				topz = @flt1 + @turn1v + @flt2 + @turn2v
                end				
			    tr = Geom::Transformation.rotation([0,0,0],[0,0,1],180.degrees)
			    tl = Geom::Transformation.translation([-@flt2*@tread-55.mm,@flt1*@tread+52.mm,topz*@riser])
			    tt = tl * tr
			    @ft3_grp.transform! tt
				if (@flt1 == 0)
				    tl = Geom::Transformation.translation([-@stg,-@nos,@stg])
                    @ft3_grp.transform! tl					
				end
				
				
			end
						
			if (@hand == "Right")
				ts = Geom::Transformation.scaling(-1,1,1)
			    tl = Geom::Transformation.translation([@wid,0,0])
				tt = tl * ts
				str_grp.transform! tt
            end								
			
			if (sto)
                stoo = sto.offset(xa,10)
				vecb = sto.vector_to(stoo)
				vecn = sto.vector_to(stf)
				rot = vecb.angle_between vecn
				if (vecn.y < 0)
				    rot = 360.degrees - rot
				end
				tr = Geom::Transformation.rotation(sto,[0,0,1],rot)
			    tl = Geom::Transformation.translation(sto)
				tt = tr * tl
				str_grp.transform! tt
			end
			
			@mod.commit_operation
				
		end   
			
            			
		def self.define_step
		    xa = X_AXIS; ya = Y_AXIS; za = Z_AXIS
			p0 = ORIGIN
			p1 = p0
			p2 = p1.offset(xa, @width)
			p3 = p2.offset(ya, @tread + @nos)
            p4 = p3.offset(xa, -@width)					
			@def_list = @mod.definitions
			@step_def = @def_list.add "Step"
			cdents = @step_def.entities
			step = cdents.add_face(p1,p2,p3,p4)
			step.pushpull @trd
		end
		def self.define_rise
			@def_list = @mod.definitions
			@rise_def = @def_list.add "Rise"
			xa = X_AXIS; ya = Y_AXIS; za = Z_AXIS
			r0 = ORIGIN
			cdents = @rise_def.entities
			r1 = r0.offset(xa, @width)					
			r2 = r1.offset(za, @riser)
			r3 = r0.offset(za, @riser)					
			rise = cdents.add_face(r0,r1,r2,r3)
			rise.pushpull -@ris   
		end
	
		def self.winders
            @wbox_grp = @sents.add_group()
			@wbents = @wbox_grp.entities
			@wbox_grp.name = "winderbox1"
			xa = X_AXIS; ya = Y_AXIS; za = Z_AXIS
			w0 = ORIGIN
			w1 = w0.offset(xa, @width)
			w2 = w1.offset(ya,@scf * 503.5.mm)
			w3 = w0.offset(xa, (75.mm - @stg) / 2)
			w3.offset!(ya, 70.mm)
			w4 = w3.offset(ya, -38.mm)
			w5 = w4.offset(xa, -(75.mm - @stg) / 2)
			kite1_grp = @wbents.add_group()
			kite1_grp.name = "Kite1"
			k1ents = kite1_grp.entities
			kite1 = k1ents.add_face(w0,w1,w2,w3,w4,w5)
			kite1.pushpull -@trd

			pw1 = @pt.offset(ya, - @nos)
			pw1.offset!(za, @riser)
			kite1_grp.move! pw1

			pr1 = pw1.offset(ya, @nos)
			pr1 = pr1.offset!(za, -@riser)
					
			inst = @wbents.add_instance @rise_def, pr1

			r1 = w0.offset(xa, (75.mm - @stg) / 2)
			r1.offset!(ya, 70.mm)
			r2 = w0.offset(xa, @width)
			r2.offset!(ya, @scf * 503.5.mm)
			r3 = r2.offset(ya, 13.6.mm)
			r4 = w0.offset(xa, (75.mm - @stg) / 2)
			r4.offset!(ya, 83.6.mm)
			risek1_grp = @wbents.add_group()
			risek1_grp.name = "KiteRiser1"
			rk1ents = risek1_grp.entities
			risek1 = rk1ents.add_face(r1,r2,r3,r4)
			risek1.pushpull -@riser
			risek1_grp.move! pw1
					
			w1 = w0.offset(xa, 21.5.mm)
			w1.offset!(ya, 47.3.mm)
			w2 = w0.offset(xa, @width)
			w2.offset!(ya, @scf * 480.8.mm)
			v2 = w2.offset(za, 50.mm)
			w3 = w0.offset(xa, @width)
			w3.offset!(ya, @width + 85.5.mm)
			v3 = w3.offset(za, 100.mm)
			w5 = w0.offset(ya, 107.mm)
			w5.offset!(xa, 16.5.mm)
			w4 = w0.offset(ya, @width + 85.5.mm)
			w4.offset!(xa, @scf * 495.mm)
			w6 = w5.offset(xa, 5.mm)
			kite2_grp = @wbents.add_group()
			kite2_grp.name = "Kite2"
			
			k2ents = kite2_grp.entities
			kite2 = k2ents.add_face(w1,w2,w3,w4,w5,w6)
			kite2.pushpull -@trd
			pw1.offset!(za, @riser)
			kite2_grp.move! pw1
			
			r1 = w0.offset(xa, 16.5.mm)
			r1.offset!(ya, 107.mm)
			r2 = w0.offset(ya, @width + 85.5.mm)
			r2.offset!(xa, @scf * 495.mm)
			r3 = r2.offset(xa, -13.9.mm)
			r4 = w0.offset(xa, 2.6.mm)
			r4.offset!(ya, 107.mm)
			risek2_grp = @wbents.add_group()
			risek2_grp.name = "KiteRiser2"
			rk2ents = risek2_grp.entities
			risek2 = rk2ents.add_face(r1,r2,r3,r4)
			risek2.pushpull -@riser
			risek2_grp.move! pw1
					
			w1 = w0.offset(xa, (75.mm - @stg) / 2)
			w1.offset!(ya, 76.mm)
			w2 = w1.offset(ya, 31.mm)
			w3 = w2.offset(xa, -75.mm)
			w4 = w3.offset(ya, @width -21.5.mm)
			v5 = w4.offset(za, 268.3.mm)
			w5 = w0.offset(xa, @scf * 518.2.mm)
			w5.offset!(ya, @width + 85.5.mm)
			v4 = w5.offset(za, 50.mm)
			kite3_grp = @wbents.add_group()
			kite3_grp.name = "Kite3"
			k3ents = kite3_grp.entities
			kite3 = k3ents.add_face(w1,w2,w3,w4,w5)
			kite3.pushpull -@trd
			pw1.offset!(za, @riser)
			kite3_grp.move! pw1

			w1 = w0.offset(xa, (75.mm - @stg) / 2)
			w1.offset!(ya, 32.mm)
			w2 = w1.offset(ya, 75.mm)
			w3 = w2.offset(xa, -75.mm)
			w4 = w3.offset(ya, -75.mm)
			kite5_grp = @wbents.add_group()
			kite5_grp.name = "Post"
			k5ents = kite5_grp.entities
			kite5 = k5ents.add_face(w1,w2,w3,w4)
			kite5.pushpull @riser * 5
			pw1.offset!(za, @riser)
			kite5_grp.move! pw1
			pp1 = pw1.offset(za, 100.mm)
			kite5_grp.move! pp1
			@pt2 = pw1
			if (@strgon == "On")
				w1 = w0.offset(xa, @width)
				w1.offset!(ya, 32.mm)
				w1.offset!(za, 112.mm)
				w2 = w1.offset(ya, @scf * 449.1.mm)
				w2.offset!(za, 216.7.mm)
				w3 = w0.offset(xa, @width)
				w3.offset!(ya, @width + @stg + 85.5.mm)
				w3.offset!(za, 412.mm) #83
				w4 = w3.offset(za, -323.mm)
				w5 = w2.offset(za, -323.mm)
				w6 = w1.offset(za, -323.mm)

				kite6_grp = @wbents.add_group()
				kite6_grp.name = "CornerString"
				k6ents = kite6_grp.entities
				kite6 = k6ents.add_face(w1,w2,w3,w4,w5,w6)
				kite6.pushpull -@stg

				w1= w0.offset(ya, @width + 85.5.mm)
				w1.offset!(xa, -53.5.mm)
				w1.offset!(za, 701.5.mm)
				w2 = w1.offset(xa, @scf * 571.7.mm)
				w2.offset!(za, -218.3.mm)
				w3 = w1.offset(xa, @width + 53.5.mm)
				w3.offset!(za, -289.5.mm) #121.3
				w4 = w3.offset(za, -323.mm)
				w5 = w2.offset(za, -323.mm)
				w6 = w1.offset(za, -323.mm)
				kite7 = k6ents.add_face(w1,w2,w3,w4,w5,w6)
				kite7.pushpull @stg
			
				pcs = pw1.offset(za, -617.9.mm)
				kite6_grp.move! pcs
			end
 
		end
		
		
	end	
end					
					

unless file_loaded?("db1stairbuilder")
	UI.menu("Plugins").add_item("db Stair Builder") { DeanBy7::Stair_Builder.main }
	file_loaded("db1stairbuilder")
end	
